@@ -44,8 +44,14 @@ const	char*	not_found = "\'%s\' is not found!\n";
 const	char*	read_error = "\'%s\' read error!\n";
 const	char*	write_error = "\'%s\' write error!\n";
 const	char*	gz_unsupport = "compressed %s is not supported!\n";
+const	char*	bad_file = "Bad binary file : %s !\n";
+const	char*	gnm2tab = "gnm2tab";
+const	char*	esc_code = "\x1b[";
+const	char*	font_end = "</font></b>";
+
+		//lpw blk Nout Ncolony eij ovl fnm rm trim lg lbl dsc odr spj olr color self term
+OUTPRM	OutPrm = {60, 0, 16, 1, 4, 10, 5, 0, 1, 0, 3, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0};
 
-static	const	char*	gnm2tab = "gnm2tab";
 static  const   char*   font_tag[3] = {
 	"<b><font color=\"%s\">",
 	"<b><font style=\"background-color:%s\">",
@@ -60,7 +66,7 @@ static	int	raw_scn(char* str, char* format, va_list args);
 static	void	display(const char* s, va_list args);
 
 #if LEAKTRACE
-LeakTrace	leak_trace;
+//leaktracer::LeakTrace	leak_trace;
 #endif
 
 char* fgets_wocr(char* str, INT maxl, FILE* fd)
@@ -112,14 +118,14 @@ char* makefnam(const char* fnam, const char* defn, char* result)
 	char const*	sg[5];
 	char const*	sd[5];
 	char	buf[LINE_MAX];
-	int 	i, l;
 
 	if (fnam == result) fnam = strcpy(buf, fnam);
 	make_fn(fnam, sg);
 	make_fn(defn, sd);
 	*result = '\0';
-	for (i = 0; i < 4; i++) {
-	    if ((l = sg[i+1] - sg[i]))
+	for (int i = 0; i < 4; ++i) {
+const	    int	l = sg[i+1] - sg[i];
+	    if (l)
 		strncat(result, sg[i], l);
 	    else
 		strncat(result, sd[i], sd[i+1] - sd[i]);
@@ -141,25 +147,26 @@ char* partfnam(char* part, const char* fname, const char* where)
 	    else
 		while (*fname++ != ':');
 	}
-	while (strchr(fname, PATHDELM))
+	while (strchr(fname, PATHDELM)) {
 	    if (strchr(where, 'p'))
 		while ((*part++ = *fname++) != PATHDELM);
 	    else
 		while (*fname++ != PATHDELM);
-	if (strchr(fname, '.')) {
+	}
+const	char*	dot = strrchr(fname, '.');
+	if (dot) {
 	    if (strchr(where, 'b'))
-		while (*fname != '.') *part++ = *fname++;
+		while (fname != dot) *part++ = *fname++;
 	    else
-		while (*fname != '.') fname++;
+		while (fname != dot) fname++;
 	    if (strchr(where, 'e')) {
 		while ((*part++ = *fname++));
 	    } else
 		*part = '\0';
-	}
-	else {
-	    if (strchr(where, 'b')) {
+	} else {
+	    if (strchr(where, 'b'))
 		while ((*part++ = *fname++));
-	    } else *part = '\0';
+	    else *part = '\0';
 	}
 	return (top);
 }
@@ -174,6 +181,19 @@ size_t file_size(FILE* fd)
 	return (fs);
 }
 
+// sizeof(buf) must be larger than strlen(fn) + strlen(ext)
+
+char* add_ext(const char* fn, const char* ext, char* buf)
+{
+	strcpy(buf, fn);
+	if (!ext || !*ext) return (buf);
+	if (*ext == '.') ++ext;
+	char*	dot = strrchr(buf, '.');
+	if (dot && !strcmp(dot + 1, ext)) return (buf);
+	if (!dot || strlen(dot) > 1) strcat(buf, ".");
+	return (strcat(buf, ext));
+}
+
 FILE* fopenpbe(const char* path, const char* name, 
 	const char* extent, const char* opt, int lvl, char* str)
 {
@@ -209,6 +229,19 @@ const 	char*	pt = path? path: "";
 	return (0);
 }
 
+FILE* wfopen(const char* name, const char* mode)
+{
+	if (is_file(name)) {
+	    int	c = 'y';
+	    if (OutPrm.overwrite == 0)
+		c = progetc("Overwrite existing file \"%s\"? [y/n] ", name);
+	    else if (OutPrm.overwrite > 1)
+		c = 'n';
+	    if (tolower(c) != 'y') return (0);
+	}
+	return (fopen(name, mode));
+}
+
 FILE* qopen(const char* dfname, const char* mode)
 {
 	char	fname[LINE_MAX];
@@ -294,7 +327,6 @@ FILE*	Ftable::fopen(const char* fname, const char* mode)
 		if (fd) return (fd);
 	    }
 	}
-	fatal("%s not found! Confirm whether ALN_TAB is correctly set!\n", fname);
 	return (0);
 }
 
@@ -406,7 +438,6 @@ Ftable ftable;
 *
 *********************************************************************/
 
-
 static INTERACT crt = {1, 0};
 
 void setprompt(int prom, int ech)
@@ -565,17 +596,6 @@ static void display(const char* s, va_list args)
 	}
 }
 
-void fatal(const char* format,...)
-{
-	va_list	args;
-
-	va_start(args, format);
-	(void) vfprintf(stderr, format, args);
-	putc('\n', stderr);
-	va_end(args);
-	exit(1);
-}
-
 void prompt(const char* format,...)
 {
 	va_list	args;
@@ -831,6 +851,22 @@ int Gnm2tab::taxon_code(const char* sqid, char** taxon)
 
 #if USE_ZLIB
 
+gzFile wgzopen(const char* name, const char* mode)
+{
+	char	str[LINE_MAX];
+	strcpy(str, name);
+	if (!is_gz(name)) strcat(str, gz_ext);
+	if (is_file(str)) {
+	    int	c = 'y';
+	    if (OutPrm.overwrite == 0)
+		c = progetc("Overwrite existing file \"%s\"? [y/n] ", str);
+	    else if (OutPrm.overwrite > 1)
+		c = 'n';
+	    if (tolower(c) != 'y') return (0);
+	}
+	return (gzopen(str, mode));
+}
+
 gzFile gzopenpbe(const char* path, const char* name, 
 	const char* extent, const char* opt, int lvl, char* str)
 {
@@ -851,12 +887,11 @@ const 	char*	pt = path? path: "";
 	    if (ps > str && ps[-1] != PATHDELM)
 		*ps++ = PATHDELM;
 	    *ps = '\0';
+	    strcat(str, name);
 	    if (extent) {
-		partfnam(str + strlen(str), name, "b");
 		if (*extent && *extent != '.') strcat(str, ".");
 		strcat(str, extent);
-	    } else
-		strcat(str, name);
+	    }
 	    if ((fd = gzopen(str, opt))) return (fd);
 	} while (*pt);
 
@@ -866,5 +901,76 @@ const 	char*	pt = path? path: "";
 	return (0);
 }
 
-#endif
+size_t lgzread(gzFile gzfd, char* buf, size_t sz)
+{
+	char*	tq = buf + sz;
+	for (char* sq = buf; sq < tq; sq += INT_MAX) {
+const	    int	n = std::min(long(INT_MAX), tq - sq);
+	    int	actural_read = gzread(gzfd, sq, n);
+	    if (actural_read <= 0) {
+		int	z_errnum = 0;
+		fatal("gz_read_error %s!\n", gzerror(gzfd, &z_errnum));
+	    }
+	}
+	return (sz);
+}
+
+size_t lgzwrite(gzFile gzfd, const char* buf, size_t sz)
+{
+const	char*	tq = buf + sz;
+	for (const char* sq = buf; sq < tq; sq += INT_MAX) {
+const	    int	n = std::min(long(INT_MAX), tq - sq);
+	    int	actural_write = gzwrite(gzfd, sq, n);
+	    if (actural_write <= 0) {
+		int	z_errnum = 0;
+		fatal("gz_write_error %s!\n", gzerror(gzfd, &z_errnum));
+	    }
+	}
+	return (sz);
+}
+
+gzFile Ftable::gzopen(const char* fname, const char* mode)
+{
+	char	str[LINE_MAX];
+	gzFile	fd = 0;
+const	char*	ext = is_gz(fname)? 0: gz_ext;
+	for (int i = 0; i < n_tabpath; ++i) {
+	    if (tabpath[i]) {
+		strcpy(str, tabpath[i]);
+		if (subdir) {
+		    strcat(str, "/");
+		    strcat(str, subdir);
+		    fd = gzopenpbe(str, fname, ext, mode, -1);
+		    if (fd) return (fd);
+		}
+		fd = gzopenpbe(tabpath[i], fname, ext, mode, -1);
+		if (fd) return (fd);
+	    }
+	}
+	fatal("%s not found! Confirm whether ALN_TAB is correctly set!\n", fname);
+	return (0);
+}
+
+gzFile Ftable::gzopen(const char* fname, const char* envpath, const char* defpath)
+{
+	gzFile	fd = this->gzopen(fname, "r");
+	if (fd) return (fd);
+const	char*	ext = is_gz(fname)? 0: gz_ext;
+
+	char	str[LINE_MAX];
+	char*	path = envpath? getenv(envpath): 0;
+	if (path) {
+	    strcpy(str, path);
+	    fd = gzopenpbe(str, fname, ext, "r", -1);
+	    if (fd) return (fd);
+	}
+	if (defpath) {
+	    strcpy(str, defpath);
+	    fd = gzopenpbe(str, fname, ext, "r", -1);
+	    if (fd) return (fd);
+	}
+	return (0);
+}
+
+#endif	// USE_ZLIB
 
